import random
from fractions import Fraction
from math import *
from operator import add,sub, mul
from copy import copy,deepcopy
from adem.tools.exprs import *
from adem.errors.exceptions import *
from adem.fundamentals import Num, whole
from adem.misc.gen import intable
try:
    from numpy import array,ndarray
except TypeError:
    pass


class Matrix:
    '''
        Calling this class create a Matrix Object.
        The class will initialize a Matrix for you by just calling Matrix()
        if you want to convert arrays to Matrix Object you use the X parameter
            You convert a list to Matrix Object; this will work fine if the
                first 2 elements gives the order of the matrix

                e.g To convert [1,2,3,4,5,6,7,8], this can be a 1x8, 2x4, 4x2 or 8x1
                    so you indicate the order by adding the order to the begining
                    if the order is 2 x 4
                    Mat_A = Matrix(X = [2,4,1,2,3,4,5,6,7,8]
                    
            You can convert 2D array (in form of list)
                e.g Mat_A = Matrix([[1,2],[3,4],[5,6],[7,8]])

            You can convert numpy arrays as well
                e.g if Y is an numpy.ndarray object, then Mat_A = Matrix(X = Y)

    '''
            
    def __init__(self, X = []):
        if X != []:
            Mat = X
            if isinstance(X,ndarray):
                Mat = list(X.shape)
                for rows in X:
                    Mat += list(rows)
            elif X and isinstance(X[0],list):
                Mat = [len(X),len(X[0])]
                for rows in X:
                    if len(rows) != Mat[1]:
                        raise DimensionError('Dimension Inconsistent')
                    Mat += rows
        else:
            Mat = []
            try:
                order = input("Matrix Order: ")
                col = int(order.split('x')[1]); row = int(order.split('x')[0])
                Mat.append(row); Mat.append(col); count = 1
                while count <= row:
                    disp = "Enter the"
                    if count == 1:
                        pos = 'First'
                    elif count ==2:
                        pos = 'Second'
                    elif count == row:
                        pos = 'Last'; disp = 'Good, Now the'
                    else: pos = 'Next'; disp = ''
                    element = input(f"{disp} {pos} row: ").split()
                    while True:
                        if len(element) == col:
                            for elements in element:
                                Mat.append(eval(elements))
                            break
                        num = "element" if len(element) == 1 else 'elements'
                        print(f"Error, You Entered {len(element)} {num} instead of {col}")
                        element = input("Try Again: ").split()
                    count += 1
                            
            except Exception:
                print("Error")
        self.Mat = Mat
        self.state = 'enabled'
    def __str__(self,pr_sr = True):
        for integers in range(self.rows()):
            line = []
            # Checking if elements are whole number and convert to fractions if not
            for integer in range(self.cols()):
                if intable(self[integers + 1,integer + 1]):
                    line.append(int(self[integers + 1,integer + 1]))
                else: 
                    frac = Fraction(self[integers + 1,integer + 1]).limit_denominator()
                    line += [frac.numerator] if frac.denominator == 1 else [f'{frac.numerator}/{frac.denominator}']
            if pr_sr:
                print(line)
        return ''
    def null(self,order):
        Mat = order
        mat = [0 for i in range(order[1]*order[0])]
        return Matrix(Mat + mat)
    def identity(self,order):
        Mat = order
        if Mat[0] != Mat[1]:
            raise OrderError
        for integer in range(order[0]):
            for integers in range(order[1]):
                Mat.append(1) if integer == integers else Mat.append(0)
        return Matrix(Mat)
    def __getitem__(self,other):
        if isinstance(other,int): # To get elements element-wise
            return self.Mat[other+1]
        elif len(other) == 2:# To get element located i,j
            if other[1] > self.cols() or other[0] > self.rows():
                raise OutOfRange(f"This Matrix only has {self.rows()} rows and {self.cols()} columns")
            return self.get_row(other[0])[other[1]]
    def __delitem__(self,key):
        # check if Matrix is active or immutable
        if self.isdisable():
            raise OperationNotAllow('This Matrix is Disabled for Modification')
        if self.isimut():
            raise InvalidOperation('This Matrix is Immutable')
        # if key is an integer rows is deleted else if key is a string column is deleted
        if type(key) == int:
            if key > self.rows():
                raise OutOfRange(f"This Matrix only has {self.rows()} rows, Hence Can't get to row {key}")
            if key < 0:
                key = self.rows() + 1 + key
            for integer in range(self.cols()):
                del self.Mat[(key-1)*self.cols()+2]
            self.Mat[0] -= 1
        elif type(key) == str:
            if int(key) > self.cols():
                raise OutOfRange(f"This Matrix only has {self.cols()} columns, Hence Can't get to column {key}")
            if int(key) < 0:
                key = self.cols() + 1 + int(key)
            for integer in range(self.rows()):
                del self.Mat[integer*self.cols()+2+int(key)-(integer+1)]
            self.Mat[1] -= 1
        elif len(key) == 2:
            raise InvalidOperation("Can only delete rows or columns not single item")
    def __shield__(self):
        self.state = 'shield'
    def __ishield__(self):
        self.state = 'active'
    def __disable__(self):
        self.state = 'disabled'
    def __enable__(self):
        self.state = 'enabled'
    def pop(self,key=1):
        # if key is an integer rows is returned else if key is a string column is returned
        if type(key) == int:
            if key < 0:
                key = self.rows() + 1 + key
            get = self.get_row(key)
            del self[key]
        elif type(key) == str:
            if int(key[1]) < 0:
                key = self.cols() + 1 + int(key)
            get = self.get_col(int(key))
            del self[str(key)]
        return get
    def __setitem__(self,key,value):
        # check if Matrix is active or immutable
        if self.isdisable():
            raise OperationNotAllow('This Matrix is Disabled for Modification')
        if self.isimut():
            raise InvalidOperation('This Matrix is Immutable')
        # row for integer, col for string, i,j for list 
        if type(key) == int:
            if key < 0:
                key = self.rows() + 1 + key
            if type(value) != list:
                self.Mat[key+1] = eval(f'{value}')
            elif type(value) == list:
                if key == self.rows()+1:
                    self.add_row([value])
                elif key > 0 and key <= self.rows():
                    self.add_row([value])
                    self.perform('~',rows=[-1, key])
                    del self[-1]
        elif isinstance(key,str):
            self.__setitem__(['*',int(key)],value)
        elif len(key) == 2:
            key = list(key)
            if key[0] == '*':
                if key[1] < 0:
                    key = ['*',self.cols() + 1 + key[1]]
                if key[1] == self.cols()+1:
                    self.add_col([value])
                elif key[1] > 0 and key[1] <= self.cols():
                    self.add_col([value])
                    self.perform('~',cols=[-1, key[1]])
                    del self['-1']
            elif key[0] > self.rows() or key[1] > self.cols():
                raise OutOfRange(f"Entry {key} doesn't exist. This Matrix only has {self.rows()} rows and {self.cols()} columns")
            else:
                if key[1] < 0:
                    key[1] += self.cols() + 1
                if key[0] < 0:
                    key[0] += self.rows() + 1
                self.Mat[(key[0]-1)*self.cols()+ key[1]+1] = eval(f'{value}')
    def __eq__(self,other):
        return self.Mat == other.Mat
    def __ne__(self,other):
        return self.Mat != other.Mat
    def __lt__(self,other):
        if isinstance(other,Matrix):
            other = other.rank()
        return self.rank() < other
    def __le__(self,other):
        if isinstance(other,Matrix):
            other = other.rank()
        return self.rank() <= other
    def __gt__(self,other):
        if isinstance(other,Matrix):
            other = other.rank()
        return self.rank() > other
    def __ge__(self,other):
        if isinstance(other,Matrix):
            other = other.rank()
        return self.rank() >= other
    def __rlt__(self,other):
        return self > other
    def __le__(self,other):
        return self >= other
    def __gt__(self,other):
        return self < other
    def __ge__(self,other):
        return self <= other
    def __bool__(self):
        if self != mat.null(len(self)):
            return True
        return False
    def rank(self):
        rank_ = 0
        for rows in self.echelon():
            for numbers in rows:
                if numbers != 0:
                    rank_ += 1
                    break
        return rank_
    class mat_iter:
        def __init__(self,Mat):
            self.n = 1
            self.stop = Mat.rows()
            self.step = 1
            self.Mat = Mat
        def __next__(self):
            if (self.n > self.stop):
                raise StopIteration
            row = self.Mat.get_row(self.n)
            self.n += self.step
            return row
        def __iter__(self):
            return self
    def __iter__(self):
        return Matrix.mat_iter(self)
    def __copy__(self):
        mat_ = tuple(self.Mat)
        mat_ = list(mat_)
        mat_ += [1]
        del mat_[-1]
        return Matrix(mat_)
    def __abs__(self):
        if not self.issq(): # For Square Matrix certainty
            raise InvalidOperation('Determinats are only supported by Square Matrices')
        mat_init = tuple(self.Mat)
        res = 0
        if len(self) == [2,2]:
            return self[1]*self[4] - self[3]*self[2]
        mult = self.pop()
        mat_ = tuple(self.Mat)
        for integers in range(self.cols()):
            self.Mat = list(mat_)
            del self[f'{integers+1}']
            res += (-1)**integers * mult[integers] * abs(self)
        self.Mat = list(mat_init)
        return (res)
    def get_row(self,row = 1):
        if row > self.rows():
            raise OutOfRange(f"This Matrix only have {self.rows()}")
        if row < 0: # To support negative keys
            row = self.rows() + 1 + row
        return [self.Mat[(row-1)*self.cols()+integer+2] for integer in range(self.cols())]
    def get_col(self, col = 1):
        if col < 0:
            col = self.cols() + 1 + col
        return [self.Mat[(col-1)+self.Mat[1]*intger+2] for intger in range(self.rows())]
    def get_diag(self, diag = ''):
        Diag = []
        if not self.issq():
            raise DimensionError('Can only get Diagonal of Square Matrix')
        if not diag:
            for rows in self:
                Diag.append(rows[len(Diag)])
        else:
            for rows in self:
                Diag.append(rows[self.rows()-(len(Diag)+1)])
        return Diag
    def add_row(self, row):
        for rows in row:
            if len(rows) == self.cols():
                self.Mat += rows
                self.Mat[0] += 1
    def add_col(self, col):
        addded_mat = self.Mat
        addded_mat= Matrix(addded_mat).trn()
        addded_mat.add_row(col)
        self.Mat = addded_mat.trn().Mat
    def perform(self, operation = '',rows = [],A = 1,B = 1, cols = []):
        # Row Operations
        if self.isimut():
            raise InvalidOperation('This Matrix is Immutable')
        if self.isdisable():
            raise OperationNotAllow('This Matrix is Disabled for Modification')
        # To translate to Column Operations
        if not rows:
            col_mat = Matrix(self.trn().Mat)
            col_mat.perform(operation = operation,rows = cols,A = A,B = B, cols = [])
            self.Mat = col_mat.trn().Mat
            return None
        # Row multiplication with scalar rows[0],Interchanging Rows, Addition, Subtraction
        if not operation:
            row = self.get_row(rows[1])
            for integer in range(self.cols()):
                self[rows[1],integer+1] = row[integer]*rows[0]
        elif operation == '~':
            row1 = self.get_row(rows[0])
            row2 = self.get_row(rows[1])
            for integer in range(self.cols()):
                self[rows[0],integer+1] = row2[integer]
                self[rows[1],integer+1] = row1[integer]
        elif operation == '+':
            row1 = self.get_row(rows[0])
            row2 = self.get_row(rows[1])
            for integer in range(self.cols()):
                self[rows[0],integer+1] = A*row1[integer] + B*row2[integer]
        elif operation == '-':
            row1 = self.get_row(rows[0])
            row2 = self.get_row(rows[1])
            for integer in range(self.cols()):
                self[rows[0],integer+1] = A*row1[integer] - B*row2[integer]
    def modal(self):
        # Modal Matrix
        self_ = copy(self)
        mod_mat = len(self)
        zeros = [0 for zeros in range(self.rows())]
        alpha = ['a','b','c','d','e',
                 'f','g','h','i','j',
                 'k','l','m','n','o',
                 'p','q','r','s','t',
                 'u','v','w','x','y',
                 'z']
        for evs in self.ev():
            mod = self - evs * mat.identity(len(self))
            if len(self) == [2,2]:
                mod_mat += whole([1,-mod[1]/mod[2]]) # Scaling the eign-vectors
            else:
                num = 0;sing = []
                for count,rows in enumerate(mod):
                    if num == self.rows() -1:# To ensure the number of eqns is not more than the Order less than 1
                        break
                    if rows == zeros:# To skip null rows
                        continue
                    eqn = E('0')
                    for cout,element in enumerate(rows):
                        if not cout or not element:
                            if not cout:
                                sing.append(-element)
                            continue
                        eqn += f'{element}{alpha[cout]}'
                    if not count:
                        mod_eqn = Eqn(eqn)
                    else: mod_eqn.add(eqn)
                    num += 1
                cols = [1]
                for var, values in (mod_eqn & sing).items():
                    cols.append(values)
                mod_mat += whole(cols) # Scaling the eign-vectors
        return Matrix(mod_mat).trn()
    def type(self):
        attr = []
        matt = tuple(self.Mat)
        self_ = copy(self)
        if not self_:
            attr.append('Null')
        try:
            if self_ == mat.identity(len(self)):
                attr.append('Unit')
        except Exception:
            pass
        try:
            test = self_*self_.trn()
            if test == mat.identity(len(test)):
                attr.append('Orthogonal')
        except Exception:
            pass
        self_ = copy(self)
        try:
            if self_** 2 == self_:
                attr.append('Idempotent')
        except Exception:
            pass
        try:
            if abs(self_) == 0:
                attr.append('Singular')
        except Exception:
            pass
        self_ = copy(self)
        if len(self_)[0] == len(self_)[1]:
            attr.append('Square')
        else: attr.append('Augmented')
        if self_ == self_.trn():
            attr.append('Symmetric')
        if self_ == (-1) * self_.trn():
            diag = self.get_diag(); num = 0
            for nums in diag:
                if nums != 0:
                    num += 1
            if num:
                attr.append('Skew Symmetric')
        try:
            if self_**2 == mat.identity(len(self_)):
                attr.append('Involuntary')
        except Exception:
            pass
        self_ = copy(self)
        try:
            for i in range(2,20):
                self_ *= self
                if not self_:
                    attr.append('Nilpotent');break
        except Exception:
            pass
        self_ = copy(self)
        try:
            self_ *= self
            for i in range(3,20):
                self_ *= self
                if self_ == self:
                    attr.append('Periodic');break
        except Exception:
            pass
        self.Mat = list(matt)
        return ', '.join(attr) + ' Matrix'
    def random(self,order):
        S = self.null(order)
        for row in range(S.rows()):
            for col in range(S.cols()):
                S[row+1,col+1] = random.randint(-1000,1000)
        return S
    def unify(self):
        mart = []
        for rows in self:
            mart += rows
        return mart
    def __string__(self):
        return self.__str__(False)
    def issing(self):
        return 'Singular' in self.type()
    def isnull(self):
        return 'Null' in self.type()
    def isunit(self):
        return 'Unit' in self.type()
    def isorth(self):
        return 'Orthogonal' in self.type()
    def issq(self):
        return 'Square' in self.type()
    def isidemp(self):
        return 'Idempotent' in self.type()
    def isnilp(self):
        return 'Nilpotent' in self.type()
    def isperiodic(self):
        return 'Periodic' in self.type()
    def isimut(self):
        return self.state == 'shield'
    def isdisable(self):
        return self.state == 'disabled'
    def isenable(self):
        return self.state == 'enabled'
    def __len__(self):
        return[self.Mat[0], self.Mat[1]]
    def aug(self,aug_mat = ''):
        if self.isdisable():
            ra
        if self.isdisable():
            raise OperationNotAllow('This Matrix is Disabled for Modification')
        if not aug_mat:
            aug_mat = mat.identity(len(self))
        aug_mat = aug_mat.trn()
        _mat_ = self.trn()
        for rows in aug_mat:
            _mat_[_mat_.rows()+1] = rows
        self.Mat = _mat_.trn().Mat
    def cols(self):
        return len(self)[1]
    def rows(self):
        return len(self)[0]
    def __invert__(self):
        # Ensuring Matrix is Square and not singular
        if not self.issq():
            raise InvalidOperation('Inverse Property are only supported by Square')
        if self.issing():
            raise InvalidOperation('Singular Matrix has no Inverse')
        
        self.mat_ = self.Mat
        self.aug()
        emulated = mat.identity([self.rows(),self.rows()]);count = 0
        
        for integer in range(emulated.cols()): #row
            col = self.get_col(integer+1)
            for integers in range(emulated.rows()): #col
                count_ = 0
                if self[integers+1,integer+1] != emulated[integers+1,integer+1]:# To check if the elements corresponds as in Unit Matrix
                    if emulated[integers+1,integer+1] == 1: # To try convert the element to 1
                        try:
                            self.perform(rows = [1/self[integers+1, integer+1],integers+1])
                        except ZeroDivisionError:
                            # Switching elements to a non-zero elements after Multiply the row by the inverse
                            while True:
                                if count_ > count:
                                    if col[count_] != 0:
                                        self.perform('~',[integers+1,count_+1])
                                        self.perform([1/self[integers+1, integer+1]],integers+1)
                                        break
                                count_ += 1
                    else:
                        # Try setting the element to 0
                        while True:
                            try:
                                if integers > integer:
                                    self.perform(rows = [(self[integers+1, integer+1]/self[integer+1,integer+1])**(-1),integers+1])
                                    self.perform('-',[integers+1,integer+1])
                                elif integers < integer:
                                    self.perform(rows = [(self[integers+1, integer+1]/self[integer+1,integer+1]),integer+1])
                                    self.perform('-',[integers+1,integer+1])
                                break
                            except ZeroDivisionError:
                                while True:
                                    if count_ > count:
                                        if col[count_] != 0:
                                            self.perform('~',[integer+1,count_+1])
                                            break
                                    count_ += 1
            count += 1
        for integers in range(self.rows()):
            del self['1']
        self.mat = self.Mat
        self.Mat = self.mat_
        return Matrix(self.mat)
    def spectral(self):
        ev = self.ev()
        if self.issq():
            raise InvalidOperation('Only Square  Matrices can have Spectral Matrix')
        spec = mat.identity([self.rows(),self.cols()])
        for count,evs in enumerate(ev):
            spec[count + 1,count+1] = evs
        return spec
    def echelon(self):
        if self.isdisable():
            raise OperationNotAllow('This Matrix is Disabled for Modification')
        ech = copy(self.Mat)
        if self[1,1] != 1:
            col = self.get_col(1)
            if 1 in col:
                self.perform('~', rows = [1,col.index(1)+1])
            elif self[1,1] == 0:
                col.reverse()
                index = 0
                for item in col:
                    if item != 0:
                        break
                    index += 1
                self.perform('~', rows = [1,len(col)-index])
        for integers in range(self.cols()):
            for integer in range(self.rows()):
                col = self.get_col(integers+1)
                if integer > integers and self[integer+1,integers+1] != 0:
                    try:
                        lcm = Num([self[integers+1, integers+1],self[integer+1,integers+1]]).LCM()
                    except Exception:
                        try:
                            lcm = Num([int(self[integers+1, integers+1]),int(self[integer+1,integers+1])]).LCM()
                        except Exception: lcm = self[integers+1, integers+1]*self[integer+1,integers+1]
                    self.perform(rows = [lcm/self[integer+1,integers+1], integer+1])
                    self.perform(rows = [lcm/self[integers+1,integers+1], integers+1])
                    self.perform('-',[integer +1,integers+1])
                elif integer == integers and self[integer+1,integers+1] == 0:
                    index = 0
                    for item in col:
                        if index > integer and item != 0:
                            self.perform('~', rows = [integer+1,index +1])
                            break
                        index += 1
        for integers in range(self.rows()):
            for items in self.get_row(integers+1):
                if items != 0:
                    self.perform(rows = [1/items, integers+1]);break
        maat = copy(self.Mat)
        self.Mat = ech
        return Matrix(maat)
    
    def rechelon(self):
        ech = self.echelon()
        for integers in range(ech.rows()):
            for items in ech.get_row(max(range(ech.rows()))-integers+1):
                if items != 0:
                    index_ = ech.get_row(max(range(ech.rows()))-integers+1).index(items)
                    for number in range(max(range(self.rows()))-integers):
                        ech.perform('+', rows = [number +1,max(range(ech.rows()))-integers+1], B = (-1) * ech[number+1,index_+1])
                    break
        return ech
    
    def trn(self):
        rows = [self.cols(), self.rows()]
        for integer in range(self.cols()):
            rows += self.get_col(integer+1)
        return Matrix(rows)
    
    def minor(self,row = 1, col = 1):
        minor_ = copy(self)
        del minor_[row]
        del minor_[f'{col}']
        return minor_
    
    def cofactor(self,row = 1, col = 1):
        if len(self) == [2,2]:
            raise InvalidOperation("2x2 Matrices don't have cofactors")
        return abs(self.minor(row,col))*(-1)**(row+col)
    
    def ev(self):
        eigen_values = []
        if not self.issq():
            raise InvalidOperation('Determinats are only supported by Square Matrices')
        mat_init = tuple(self.Mat)
        res = E('0'); sett = 5
        try:
            self.__string__() # To check if the Matrix has its principal diagonal dopped
            for integers in range(self.rows()):
                self[integers+1,integers+1] = '' + '"' +f'{self[integers+1,integers+1]} - x' + '"' 
        except ValueError:
            sett = 0
        if len(self) == [2,2]:
            # check if called directly or indirectly
            if sett:
                roots = (E(f'{self[1]}')*self[4] - E(f'{self[3]}')*self[2]).simp() & 'x'
                self.Mat = list(mat_init)
                while True:
                    try:
                        eigen_values.append(next(roots))
                    except StopIteration:
                        break
                return eigen_values
            else:
                self.Mat = list(mat_init)
                return (E(f'{self[1]}')*self[4] - E(f'{self[3]}')*self[2]).simp()
        mult = self.pop()
        mat_ = tuple(self.Mat)
        for integers in range(self.cols()):
            self.Mat = list(mat_)
            del self[f'{integers+1}']
            res += (E(f'{mult[integers]}') * self.ev()).simp()
        self.Mat = list(mat_init)
        # check if called directly or indirectly
        if not sett:
            return res.simp()
        roots = res.simp() & 'x'
        while True:
            try:
                eigen_values.append(next(roots))
            except StopIteration:
                break
        return eigen_values
    def new(self,new_mat):
        self.Mat = new_mat
        return self
    def cofactors(self):
        cofactor = [self.rows(),self.cols()]
        for integer in range(self.rows()):
            for integers in range(self.cols()):
                cofactor.append(self.cofactor(integer+1,integers+1))
        return Matrix(cofactor)
    def property(self):
        print(self)
        try:
            print(self.type())
            try:
                print(f'Determinat is {abs(self)}')
            except InvalidOperation:
                print('Determinant Not Supported')
            print(f'Rank is {self.rank()}')
            try:
                if self.isnilp():
                    self_ = copy(self)
                    for i in range(2,20):
                        self_ *= self
                        if not self_:
                            print(f'This Matrix is Nilpotent to the index of {i+1}')
                            break;
            except Exception:
                pass
            try:
                if self.isperiodic():
                    self_ = copy(self)
                    self_ *= self_
                    for i in range(3,20):
                        self_ *= self
                        if not self_ == self:
                            print(f'This Matrix has a period of {i-1}')
                            break
            except Exception:
                pass
            self_ = copy(self)
            try:
                for i in range(2,20):
                    self_ *= self_
                    if self_ == ~self_:
                        print(f'This Matrix raised to the power of {i} gives its inverse\n Hence This Matrix raised to the power of {i+1} gives a Unit Matrix')
                        break;
            except Exception:
                pass
            print('------Transpose-------')
            print(self.trn())
            print('------Cofactors-------')
            try:
                print(self.cofactors())
            except InvalidOperation:
                print('Not Supported')
            print('------Spectral-------')
            print(self.spectral())
            print('------Modal-------')
            print(self.modal())
            print('------Inverse-------')
            try:
                print(~self)
            except InvalidOperation:
                print('Singular Matrix has no Inverse')
            print('------Echelon Form-------')
            print(self.echelon())
            print('------Reduced Echelon Form-------')
            print(self.rechelon())
        except OperationNotAllow:
            print('This Matrix is Disabled')
        
    def adj(self):
        return self.cofactors().trn()
    def __add__(self,other):
        if len(self) != len(other):
            raise DimensionError(f'Incompatible Matrices: {" x ".join(len(self))} and {" x ".join(len(other))}')
        new = len(self)
        for integer in range(self.rows()):
            new += list(map(add, self.get_row(integer+1), other.get_row(integer+1)))
        return Matrix(new)
    def __sub__(self,other):
        if len(self) != len(other):
            raise DimensionError(f'Incompatible Matrices: {" x ".join(len(self))} and {" x ".join(len(other))}')
        new = len(self)
        for integer in range(self.rows()):
            new += list(map(sub, self.get_row(integer+1), other.get_row(integer+1)))
        return Matrix(new)
    def __mul__(self,other):
        new = len(self)
        if isinstance(other, int) or isinstance(other, float):
            for integer in range(self.rows()):
                new += [other*item for integer in range(self.rows())]
        else:
            if not self.cols() == other.rows():
                raise DimensionError(f'Incompatible Matrices: {" x ".join(len(self))} and {" x ".join(len(other))}')
            for integer in range(self.rows()):
                for integers in range(self.rows()):
                    new.append(sum(list(map(mul, self.get_row(integer+1), other.get_col(integers+1)))))
        return Matrix(new)
    def __rmul__(self,other):
        return self.__mul__(other)
    
    def __pow__(self,other):
        obj = copy(self)
        if isinstance(other, int):
            for integer in range(abs(other)-1):
                obj *= self
            if other < 0:
                return ~obj
        return obj
    def __truediv__(self,other):
        if isinstance(other, int) or isinstance(other, float):
            return self * (1/other)
        return self * ~other
    def __rtruediv__(self,other):
        return other * ~A
    def __truth__(self):
        get = ''
        for integer in range(self.rows()):
            for integers in range(self.cols()):
                if self[integer+1,integers+1]:
                    return True
        return False
        
def on(object):
    object.__enable__()
def off(object):
    object.__disable__()
def sus(object):
    object.__shield__()
def res(object):
    object.__ishield__()
mat = Matrix("1 ")





from fractions import Fraction


class Num:
    def __init__(self,Num_list):
        if type(Num_list) == int:
            self.int = Num_list
        elif type(Num_list) == list:
            self.list = Num_list
        else:
            raise TypeError(Num_list + 'is not an integer')
    def is_prime(self,start = 2):
        if type(self.int) and not self.int == 1:
            num_range = range(start,self.int)
            for num in num_range:
                if self.int % num == 0:
                    return False
            return True
    def next_prime(self):
        num = self.int+1
        while not Num(num).is_prime():
            num +=1
        return num
    def is_even(self):
        if self.int % 2 == 0:
            return True
        return False
    def is_odd(self):
        if self.int % 2 == 1:
            return True
        return False
    def is_divisible(self,divisor):
        if self.int % divisor == 0:
            return True
        return False
    def FirstDivisor(self):
        if type(self.int):
            num_range = range(2,self.int+1)
            for num in num_range:
                if self.int % num == 0:
                    return num
        return 1
    def is_perfectsq(self):
        if int(self.int**.5) == self.int**.5:
            return True
        return False
    def prime_factors(self):
        factor_list =[];starter = self.int
        while True:
            divisor = Num(int(starter)).FirstDivisor()
            if not divisor in factor_list: 
                factor_list.append(divisor)
            starter /= divisor
            if starter == 1:
                break
        return Data(factor_list).to_str(', ')
    def in_prime_factors(self):
        factor_list =[];starter = self.int
        while True:
            divisor = Num(int(starter)).FirstDivisor()
            factor_list.append(divisor)
            starter /= divisor
            if int(starter) == 1:
                return Data(factor_list).to_str(' x ')
    def mult_tru(self):
        ans = 1
        for item in self.list:
            ans *= item
        return ans
    def GCD(self):
        listt = [];factor_list = []
        for item in self.list:
            listt.append(Data(Num(item).in_prime_factors()).count(['x',' ']))
        if listt:
            for item in listt[0]:
                tem = []
                for list in listt:
                    if item in list:
                        tem.append(item)
                if len(tem) == len(listt):
                    tem = []
                    for list in listt:
                        tem.append(list[item])
                    for counting in range(0,min(tem)):
                        factor_list.append(int(item))
            return Num(factor_list).mult_tru()
    def LCM(self):
        listt = [];factor_list = [];num_list=[]; tem = []
        for item in self.list:
            listt.append(Data(Num(item).in_prime_factors()).count(['x',' ']))
        if listt != []:
            for list in listt:
                for num in list:
                    if not num in num_list:
                        num_list.append(num)
            for num in num_list:
                tem = []
                for list in listt:
                    try:
                        tem.append(list[num])
                    except KeyError:
                        tem.append(0)
                for counting in range(0,max(tem)):
                    factor_list.append(int(num))
            return Num(factor_list).mult_tru()
class Data:
    def __init__(self,typ):
        if type(typ) == list:
            self.list = typ
        elif type(typ) == str:
            self.str = typ
    def to_str(self,sep = ', '):
        Str ='';counter = 1
        for item in self.list:
            Str += str(item)
            if counter != len(self.list):
                Str += sep
            counter += 1
        return Str
    def count(self,exception=[],split_=' '):
        temp = [];item_list =[]; occurence = []
        for char in self.str.split(split_):
            if not char in temp and not char in exception:
                temp.append(char)
        for item in temp:
            item_list.append(item)
            occurence.append(self.str.count(item))
        return dict(zip(item_list,occurence))
class dic:
    def __init__(self,item,index = None,Error = None,key = '',Limit = True):
        if type(item) == dict:
            self.keys = list(item)
            self.values = list(item.values())
            self.Error = Error
        elif index:
            return item[index]
        elif type(item) == list:
            self.list = item
            self.key = key
            self.Limit = Limit
        else: self.item = item
        
    def key(self,num):
        if not self.Error:
            return self.values[self.keys.index(num)]
        else:
            try:
                return self.values[self.keys.index(num)]
            except ValueError:
                return num
    def value(self,num):
        if not self.Error:
            return self.keys[self.values.index(num)]
        else:
            try:
                return self.keys[self.values.index(num)]
            except ValueError:
                return num
    def index(self,index):
        return dic(self.item,index = index)
    def search(self,index):
        occurence = 0
        while index < len(self.list):
            if self.list[index] == self.key:
                occurence+=1;index+=1
            elif self.Limit == True and self.list[index] != self.key:
                break
        return occurence




class OrderError(Exception):
    pass
class DimensionError(Exception):
    pass
class IncompatibleOrder(Exception):
    pass
class InvalidOperation(Exception):
    pass
class OperationNotAllow(Exception):
     pass
class OutOfRange(Exception):
    pass





from adem.fundamentals.primary import Num

def whole(lst):
    den = [Fraction(lsts).limit_denominator().denominator for lsts in lst]
    return [lsts * Num(den).LCM() for lsts in lst]
def intable(num):
    try:
        return True if int(float(num)) == float(num) else False
    except Exception:
        return False
def con(dic):
    res = ''
    for key, value in dic.items():
        res += f'{key}^{value}'
    return res




from fractions import Fraction
from math import *
from copy import copy,deepcopy
from adem.fundamentals.primary import Num
from adem.misc.gen import intable


__all__ = ['E','Eqn']


class E:
    def __init__(self,expr,set_ = False):
        if not set_ and (not expr or len(expr) == expr.count(' ')):
            expr = '0'
        if set_:
            self.expr = expr
            return None
        step = 0
        var,coeff,pow_ = '','','';self.coeff = [] 
        _var = {}
        _pow = []
        self.expr = {}
        expr = expr.replace('^-','#')
        if '/' in expr:
            part = expr.split('/')
            expr = E(part[0])
            for num, alg in enumerate(part):
                if not num:
                    continue
                expr /= alg
            self.expr = expr.expr
            return
        if '+' in expr:
            part = expr.split('+')
            tie = 0
            if not part[0] or part[0] == ' ':
                expr = E(part[1])
                tie = 1
            else:
                expr = E(part[0])
            for num, alg in enumerate(part):
                if num-1 < tie:
                    continue
                expr += alg
            self.expr = expr.expr
            return
        if '-' in expr:
            part = expr.split('-');tie = 0
            if not part[0] or part[0] == ' ':
                expr = E(f';{part[1]}')
                tie = 1
            else:
                expr = E(part[0])
            for num, alg in enumerate(part):
                if num-1 < tie:
                    continue
                expr -= alg
            self.expr = expr.expr
            return None
        expr = expr.replace(';','-').replace('#','^-')
        while True:
            if expr[step] == ' ':
                step += 1
                continue
            while step < len(expr) and (expr[step].isdigit() or expr[step] in ('+','-','.') or expr[step] == ' '):
                if not coeff and expr[step] == '+':
                    step += 1
                    continue
                if expr[step] == ' ':
                    step += 1
                    continue
                coeff += expr[step]
                step += 1
            self.coeff.append(coeff)
            while step < len(expr) and (expr[step].isalpha() or expr[step] in ('+','-') or expr[step] == ' '):
                if expr[step] == ' ':
                    step += 1
                    continue
                var = expr[step]
                step += 1
                try:
                    if step < len(expr) and expr[step] == '^':
                        step += 1; pow_ = ''
                        while step < len(expr) and (expr[step].isdigit()  or expr[step] in ('+','-','.')):
                            pow_ += expr[step]
                            step += 1
                    else: pow_ = 1
                except Exception:
                    pass
                if pow_ != '0':
                    _var[var] = eval(str(pow_))
                else: var = ''
            if not var:
                _var[''] = 0
            if not coeff:
                coeff = 1
            elif coeff == '-':
                coeff = '-1'
            coeff = eval(f'{coeff}')
            if coeff not in self.expr:
                self.expr[coeff] = []
            self.expr[coeff].append(_var)
            if step >= len(expr):
                break

    def __str__(self):
        disp = '';
        for coeff in self.expr:
            if coeff < 0:
                disp += f' - {abs(coeff)}' if coeff != -1 else ' - 1' if self.expr[coeff] == [{'':0}] else ' - '
            else:
                if disp:
                   disp += ' + '
                disp += f'{coeff}' if coeff != 1 else ' 1 ' if coeff == 1 and self.expr[coeff] == [{'':0}] else ''
            for count,expr_ in enumerate(self.expr[coeff]):
                if count:
                    disp += ' - 1 ' if coeff == -1 and self.expr[coeff] == [{'':0}] else f' - ' if coeff == -1 else f' - {abs(coeff)}' if coeff < 0 else f' + {coeff}' if coeff > 1 else ' + '
                for var in expr_:
                    disp += f'{var}' if var else str(abs(coeff)) if not disp else ''
                    if expr_[var] != 1 and expr_[var] != 0:
                        disp += f"^{expr_[var]}"
                count += 1
        return disp
    def __len__(self):
        elnt = 0
        for expressions in self:
            elnt += 1
        return elnt
    def str(self,dic):
        disp = ''
        for var in dic[coeff]:
            disp += f'{var}'
            if dic[coeff][var] != 1:
                disp += f"^{[coeff][var]}"
        return disp
    def __add__(self,other):
        self_ = deepcopy(self)
        sett = 0
        if isinstance(other,str) or isinstance(other,float) or isinstance(other,int):
            other = E(f'{other}')
        ## check duplicates
        if len(other) > 1:
            for expr in other:
                self_ += expr
        if len(other) == 1:
            for coeff in other.expr:
                if coeff not in self_.expr:
                    self_.expr[coeff] = []
                self_.expr[coeff].append(other.expr[coeff][0])
        return self_
    
    def __sub__(self,other):
        try:
            self_ = deepcopy(self)
            sett = 0
            if isinstance(other,str) or isinstance(other,float) or isinstance(other,int):
                other = E(f'{other}')
        ## check duplicates
            if len(other) > 1:
                for expr in other:
                    self_ -= expr
            if len(other) == 1:
                for coeff in other.expr:
                    coeff *= -1
                    if coeff not in self_.expr:
                        self_.expr[coeff] = []
                    self_.expr[coeff].append(other.expr[-1*coeff][0])
        except RuntimeError:
            self_ = E('0')
        
        return self_
    def cal(self,values = {}):
        result = 0;used={}
        for coeff in self.expr:
            for var in self.expr[coeff]:
                term = coeff
                for var_ in var:
                    if not var_:
                        continue
                    if not values:
                        if var_ not in used:
                            ex = eval(input(f'{var_}? '))
                            used.update({f'{var_}':ex})
                        else: ex = used[var_]
                    else:
                        ex = values[var_]
                    term *= ex**var[var_]
                result += term
        return result

    def lin_diff(self,var_ = 'x'):
        tmp = {}
        for expr_ in self:
            tmp_ = E(expr_)
            for coeff, var in tmp_.expr.items():
                val_=[]; pw = 0
                for var__ in var:
                    val = {}
                    for __var in var__:
                        if __var == var_:
                            pw = var__[__var]
                            if var__[__var] - 1:
                                val[var_] = var__[__var] - 1
                        else:
                            val[__var] = var__[__var]
                    val_.append(val)
                if pw *coeff:
                    if pw * coeff in tmp:
                        tmp[pw* coeff].append(val_)
                    else:
                        tmp[pw* coeff] = val_
        return E(tmp,True).simp()
                
            
    def simp(self,psort = False):
        for coeff, var in self.expr.items():
            if var == [{}]:
                self.expr[coeff] =[{'':0}]
        alpha = ['a','b','c','d','e',
                 'f','g','h','i','j',
                 'k','l','m','n','o',
                 'p','q','r','s','t',
                 'u','v','w','x','y',
                 'z']
        res = {}; sim = '';arr = {};arr_={};_arr_ = []
        for coeff,var in self.expr.items():
            for var_ in var:
                for alpha_ in alpha:
                    if alpha_ in var_:
                        arr[alpha_] = var_[alpha_]
                _arr_.append(arr);arr =  {}
            arr_[coeff] = _arr_; _arr_ = []
        sort = E('0');used = []
        for coeff, var in arr_.items():
            if var == [{}]:
                arr_[coeff] =[{'':0}]
        for alpha_ in alpha:
            for count,expre in enumerate(E(arr_,True)):
                if alpha_ in expre and not count in used:
                    if alpha_ == 'i':
                        imj =  E(expre)
                        cof = list(imj.expr)[0]
                        pw = imj.expr[cof][0]['i']
                        imj.expr[cof][0]['i']  = pw % 2
                        if pw % 2:
                            pw-=1
                            if pw >3:
                                pw = (pw % 4)
                            elif pw < 4:
                                pw = (pw % 2)
                            pw = -1 if pw else 1
                            expre = str(E(dict(zip([cof* pw],list(imj.expr.values()))),True))
                        else:
                            if pw >3:
                                pw = (pw % 4)
                            elif pw < 4:
                                pw = (pw % 2)
                            pw = -1 if pw else 1
                            expre = cof * pw
                    sort += expre
                    used.append(count)
        for count,exprs in enumerate(E(arr_,True)):
            sett = 3
            for alphas in alpha:
                if alphas in exprs:
                    sett = 0
            if sett and count not in used:
                sort += int(float(exprs)) if intable(exprs) else exprs
        arr_ = sort.expr
        for coeff, var in arr_.items():
            if not coeff:
                continue
            for var_ in var:
                if not con(var_) in res:
                    res.update({con(var_):coeff})
                else:
                    res[con(var_)] += coeff
        for coeff, var in res.items():
            if not var or not coeff:
                continue
            if var > 0 and sim:
                sim += '+'
            sim += str(int(var) if intable(var) else var)
            if coeff != '^0':
                sim += coeff
        if not sim:
            sim = '0'
        return E(sim) if psort else E(sim).__psort__()
    
    def pow(self,exprs,var = ''):
        ex = 0
        exp = E(exprs)
        for coeff,_var in exp.expr.items():
            for var_ in _var[0]:
                ex += _var[0][var_] if not var else 0 if var and var_ != var else _var[0][var_]
        return ex
    def deg(self):
        pows = []
        for exprs in self:
            pows.append(self.pow(exprs))
        return max(pows)
    def islin(self):
        return True if self.deg() == 1 else False
    def __psort__(self):
        if not self:
            return self
        self_ = self.simp(True)
        alpha = ['a','b','c','d','e',
                 'f','g','h','i','j',
                 'k','l','m','n','o',
                 'p','q','r','s','t',
                 'u','v','w','x','y',
                 'z']
        vars_ = E('0')
        pows = [];__vars__ = {};var_pt = []
        for count,exprs in enumerate(self_):
            pows.append(self.pow(exprs))
            if not self.pow(exprs):
                var_pt.append(count +1)
        pows_ = list(set(pows));pows_.sort();pows_.reverse()
        used = [];
        for memebers in pows_:
            for alphas in alpha:
                for count,items in enumerate(pows):
                    if items != memebers or count in used:
                        continue
                    if alphas in self_[count+1]:
                        vars_ +=  self_[count+1]
                        used.append(count)
        
        for var_pts in var_pt:
            sett = 6
            for alphas in alpha:
                if alphas in self[var_pts]:
                    sett = 0
            if sett:
                vars_ += self[var_pts]
        if vars_[1] == '0':
            for coeff, var in vars_.expr.items():
                if coeff:
                    __vars__[coeff] = var
        return E(__vars__,True).simp(True)
            
    class iter:
        def __init__(self,expr):
            self.expr = expr.expr
            self.it = self.get()
        def get(self):
            for coeff in self.expr:
                disp = ''
                if coeff < 0:
                    disp = f' - {abs(coeff)}' if coeff != -1 else ' - 1 ' if self.expr[coeff] == [{'':0}] else ' - '
                else:
                    disp = f'{coeff}' if coeff != 1 else ' 1 ' if coeff == 1 and self.expr[coeff] == [{'':0}] else ''
                for count,expr_ in enumerate(self.expr[coeff]):
                    if count:
                        disp = f' - {abs(coeff)}' if coeff < 0 else f'{coeff}' if coeff > 1 else ''
                    sett = 6
                    for var in expr_:
                        disp += f'{var}' if var else str(abs(coeff)) if not disp else ''
                        if expr_[var] != 1 and expr_[var] != 0:
                            disp += f"^{expr_[var]}"
                    yield disp
                    count += 1

        def __next__(self):
            return next(self.it)
        def __iter__(self):
            return self
    def __iter__(self):
        return self.iter(self)
    def __getitem__(self,other):
        for count,expr_ in enumerate(self):
            if count + 1 == other:
                return expr_
    def __truediv__(self,other):
        self_ = deepcopy(self)
        val = {};val_ = {}
        if isinstance(other,str):
            other = E(other)
        if len(other) == 1:
            val = E('0')
            for exprs in self:
                Expr = E(exprs)
                for _coeff,_var in Expr.expr.items():
                    for coeff, var in other.expr.items():
                        for var_ in var[0]:
                            try:
                                if not Expr.expr[_coeff][0][var_] -var[0][var_]:
                                    Expr.expr[_coeff][0].pop(var_)
                                else:
                                    Expr.expr[_coeff][0][var_] -= var[0][var_]
                            except KeyError:
                                Expr.expr[_coeff][0][var_] = - var[0][var_]
                    val += str(E({_coeff/coeff:[Expr.expr[_coeff][0]]},True).simp())
            
        if len(other) > 1:
            self_ = copy(self)
            Quo = E('0')
            while True:
                factor = E(self_[1])/other[1]
                sub = factor * other
                rem = (self_ - sub).simp()
                Quo += factor
                if not rem:
                    break
                self_ = rem
            val = Quo
        return round(val.simp(),5)
    def __round__(self,other):
        coeff = [round(coeff,other) for coeff in self.expr]
        vars_ = list(self.expr.values())
        return E(dict(zip(coeff,vars_)),True)
    def __pow__(self,other):
        self_ = copy(self)
        for interger in range(other-1):
            self_ *= self
        return self_
    def __rmul__(self,other):
        return self * other
    def __and__(self,other,fix = 6):
        starter = 1
        self_ = copy(self.simp())
        while True:
            try:
                root = self_.New_Raph(self_,other)
                root = int(round(root,6)) if intable(round(root,6)) else round(root,6)
                yield root
                self_ /= E(other).simp() - root
                if not self_:
                    break
            except ZeroDivisionError:
                break

    def New_Raph(self,eq, var):
        starter = 1
        while True:
            grad = self.cal({var:starter})/self.lin_diff(var).cal({var:starter})
            starter -= grad
            if not grad:
                break
        return starter
    def __mul__(self,other):
        if isinstance(other,str) or isinstance(other,int) or isinstance(other,float):
            other = E(str(other))
        sef_ = {};val_ = {};_val_= []; statr = 0
        for __exprs__ in other:
            coeff = list(E(__exprs__).expr)[0]
            var = list(E(__exprs__).expr.values())[0]
            sef = {}
            coeff_ = []; var0 = []
            for coeff__, var_ in self.expr.items():
               coeff_ += [coeff__ * coeff]
               var0.append(var_)
               sef.update(dict(zip(coeff_,var0)))
            _self = deepcopy(self)
            sef_ = E(sef,True)
            _sef_ = E({},True)
            for _var in var:
                for _coeff, vars_ in sef_.expr.items():
                    for vars__ in vars_:
                        for _vars__ in vars__:
                            if _vars__ in _var:
                                pq = vars__[_vars__] + _var[_vars__]
                                if _vars__ == 'j':
                                    val_[_vars__] = pq % 2
                                _coeff *= (-1)**pq 
                                val_[_vars__] = vars__[_vars__] + _var[_vars__]
                            else:
                                val_[_vars__] = vars__[_vars__]
                            for __vars__ in _var:
                                if __vars__ not in val_:
                                    val_[__vars__] = _var[__vars__]
                        _val_.append(val_);val_ = {}
                    _sef_.expr[int(_coeff) if intable(_coeff) else _coeff] = _val_
                    _val_ = []
            if not statr:
                add = _sef_
            else:
                add += _sef_
            statr += 1
        return add.simp()
    def __bool__(self):
        if str(self) == '0':
            return False
        return True
    def __copy__(self):
        new = {}
        for coeff,var in self.expr.items():
            new[coeff] = var
        return E(new,True)
    def parse(self):
        new_expr = {}
        for expr in self:
            expr_ = E(expr)
            for coeff, var in expr_.expr.items():
                for var_ in var[0]:
                    new_expr[var_] = coeff
        return new_expr
Es = E('1',True)



class Eqn(E):
    def __init__(self,expr):
        self.eqn1 = expr.simp() if isinstance(expr,E) else E(expr).simp()
        self.num = 1
        self.expr = self.eqn1.expr
    def add(self,eqn):
        exec(f'self.eqn{self.num +1} = eqn.simp() if isinstance(eqn,E) else Eqn("{eqn}").simp()')
        self.num += 1
    def __str__(self):
        for nomb in range(1,self.num+1):
            exec(f'print(self.eqn{nomb})')
        return ''
    def __and__(self,other):
        if self.num != len(other):
            raise InvalidOperation('Equations')
        deg = []
        for nomb in range(1,self.num+1):
            exec(f'deg.append(self.eqn{nomb}.deg())')
        alpha = ['a','b','c','d','e',
                 'f','g','h','i','j',
                 'k','l','m','n','o',
                 'p','q','r','s','t',
                 'u','v','w','x','y',
                 'z']
        if max(deg) == 1:
            var_list = []
            for alpha_ in alpha:
                for eqns in self:
                    if alpha_ in eqns.parse():
                        var_list.append(alpha_)
                    if alpha_ in var_list:
                        break
            if len(var_list) > self.num:
                raise DimensionError
            delta = []
            mat = [len(var_list),len(var_list)]
            for eqns in self:
                for var in var_list:
                    try:
                        mat.append(eqns.parse()[var])
                    except Exception:
                        mat.append(0)
                    
            delta.append(mat)
            for vars_ in var_list:
                mat = [len(var_list),len(var_list)]
                for count,eqns in enumerate(self):
                    for var in var_list:
                        try:
                            mat.append(other[count]) if vars_ == var else mat.append(eqns.parse()[var])
                        except Exception:
                            mat.append(0)
                delta.append(mat)
            res = {}
            det = abs(Matrix(delta[0]))
            for count,var in enumerate(var_list,1):
                value = abs(Matrix(delta[count]))/det
                res[var] = int(value) if intable(value) else value
            return res
    def __delitem__(self,other):
        for numbers in range(other, self.num):
            exec(f'self.eqn{numbers} = self.eqn{numbers+1}')
        exec(f'del self.eqn{self.num};self.num -= 1')
    def __getitem__(self,other):
        return eval(f'self.eqn{other}')
    def __setitem__(self,other,value):
        exec(f'self.eqn{other} = Eqn("{value}").simp()')
    class Eqn_iter:
        def __init__(self,eqn):
            self.eqn = eqn
            self.it = self.get()
        def get(self):
            for nomb in range(1,self.eqn.num+1):
                yield eval(f'self.eqn.eqn{nomb}')
        def __next__(self):
            return next(self.it)
        def __iter__(self):
            return self
    def __iter__(self):
        return self.Eqn_iter(self)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('HISTORY.md') as hf:
    HISTORY = hf.read()

setup_args = dict(
                name="adem",
                version="1.0.4",
                author="Miracle Adebunmi",
                author_email="miraclem2014@gmail.com",
                description="Giving Python Ability to Work directly on Matrix",
                long_description=long_description +'\n\n' + HISTORY,
                license = 'MIT',
                long_description_content_type="text/markdown",
                url="https://github.com/pypa/sampleproject",
                packages = find_packages(),
                classifiers = [
                                "Programming Language :: Python :: 3",
                                "License :: OSI Approved :: MIT License",
                                "Operating System :: OS Independent",
                              ],
                python_requires ='>=3.6',
                
)

requirements = [
    'numpy>=1.6.0'
    ]

if __name__ == '__main__':
    setup(**setup_args,install_requires = requirements)
